--*****************************************************************************
--*  Copyright (c) 2014-2015 by Michael Fischer (www.emb4fun.de)
--*  All rights reserved.
--*
--*  Redistribution and use in source and binary forms, with or without 
--*  modification, are permitted provided that the following conditions 
--*  are met:
--*  
--*  1. Redistributions of source code must retain the above copyright 
--*     notice, this list of conditions and the following disclaimer.
--*  2. Redistributions in binary form must reproduce the above copyright
--*     notice, this list of conditions and the following disclaimer in the 
--*     documentation and/or other materials provided with the distribution.
--*  3. Neither the name of the author nor the names of its contributors may 
--*     be used to endorse or promote products derived from this software 
--*     without specific prior written permission.
--*
--*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
--*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
--*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
--*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
--*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
--*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
--*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
--*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
--*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
--*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
--*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
--*  SUCH DAMAGE.
--*
--*****************************************************************************
--*  History:
--*
--*  19.06.2014  mifi  First version.
--*  23.05.2015  mifi  Use correct GPMC identifier.
--*  21.07.2015  mifi  Remove Logic Analyzer output and use a wb_core.
--*****************************************************************************

--*****************************************************************************
--*  DEFINE: Library                                                          *
--*****************************************************************************
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


--*****************************************************************************
--*  DEFINE: Entity                                                           *
--*****************************************************************************

entity top is
   port ( 
      -- Input clock 
      CLOCK_50      : in  std_logic;

      -- GPMC bus
      GPMC_A        : in    std_logic_vector(10 downto 0);
      GPMC_D        : inout std_logic_vector(15 downto 0);
      GPMC_CS_n     : in    std_logic;
      GPMC_REN_n    : in    std_logic;
      GPMC_WEN_n    : in    std_logic;
      GPMC_RESET_n  : in    std_logic;
      GPMC_WAIT_n   : out   std_logic;
      
      -- LEDs
      LED           : out std_logic_vector(7 downto 0)
   );
end entity top;


--*****************************************************************************
--*  DEFINE: Architecture                                                     *
--*****************************************************************************

architecture syn of top is

   --------------------------------------------------------
   -- Define all constants here
   --------------------------------------------------------

   --------------------------------------------------------
   -- Define all components which are included here
   --------------------------------------------------------

   component wb_syscon is
      port (
         -- None Wishbone
         clk_i     : in  std_logic := '0';
         reset_i   : in  std_logic := '0';
         clk_10_o  : out std_logic;

         -- Wishbone
         clk_o     : out std_logic;
         rst_o     : out std_logic
      );
   end component wb_syscon;
   
   
   component heartbeat is
      port (  
         clk_i      : in  std_logic := '0';
         counter_o  : out std_logic
      );
   end component heartbeat;
   
   
   component wb_core is
      generic (
         ADDRESS_WIDTH : integer
      );
      port ( 
   
         -- Syscon
         clk_i         : in    std_logic := '0';
         rst_i         : in    std_logic := '0';

         -- GPMC bus
         GPMC_A        : in    std_logic_vector(ADDRESS_WIDTH-1 downto 0);
         GPMC_D        : inout std_logic_vector(15 downto 0);
         GPMC_CS_n     : in    std_logic;
         GPMC_REN_n    : in    std_logic;
         GPMC_WEN_n    : in    std_logic;
         GPMC_RESET_n  : in    std_logic;
         GPMC_WAIT_n   : out   std_logic;

         -- External
         gpio0_o       : out   std_logic_vector(15 downto 0);
         gpio0_i       : in    std_logic_vector(15 downto 0) := (others => '0');
         gpio1_o       : out   std_logic_vector(15 downto 0);
         gpio1_i       : in    std_logic_vector(15 downto 0) := (others => '0')
      );
   end component wb_core;
   
   
   --------------------------------------------------------
   -- Define all local signals here
   --------------------------------------------------------

   signal wb_clk  : std_logic;
   signal wb_rst  : std_logic;
   signal clk_10  : std_logic;

   signal gpio    : std_logic_vector(15 downto 0);
     
begin

   -- Syscon
   inst_syscon : wb_syscon 
      port map ( 
         -- None Wishbone
         clk_i    => CLOCK_50,
         reset_i  => not GPMC_RESET_n,
         clk_10_o => clk_10,

         -- Wishbone
         clk_o    => wb_clk,
         rst_o    => wb_rst
      );
      
   -- Heartbeat
   inst_heartbeat : heartbeat 
      port map ( 
         clk_i     => clk_10,
         counter_o => LED(7)
      );
      
   -- Wishbone core      
   inst_wb_core : wb_core
      generic map (
         ADDRESS_WIDTH => GPMC_A'length
      )
      port map (
         -- Syscon
         clk_i        => wb_clk,
         rst_i        => wb_rst,
         
         -- GPMC bus
         GPMC_A       => GPMC_A,
         GPMC_D       => GPMC_D,
         GPMC_CS_n    => GPMC_CS_n,
         GPMC_REN_n   => GPMC_REN_n,
         GPMC_WEN_n   => GPMC_WEN_n,
         GPMC_RESET_n => GPMC_RESET_n,
         GPMC_WAIT_n  => GPMC_WAIT_n,
         
         -- External
         gpio0_o      => gpio,
         gpio0_i      => open,
         gpio1_o      => open,
         gpio1_i      => gpio
      );
      
   --------------------------------------------------------
   -- Output
   --------------------------------------------------------

   -- LED 
   LED(6 downto 0) <= gpio(6 downto 0);
            
end architecture syn;

-- *** EOF ***

